from scapy.config import conf
from scapy.contrib.cansocket_python_can import CANSocket
from scapy.main import load_contrib

from can_id_sheet import VehicleSegment
from ecu_process import GenericECU

from time import sleep
import multiprocessing as mp

import can


def coordinate_python_can():
    # create a bus instance using 'with' statement,
    # this will cause bus.shutdown() to be called on the block exit;
    # many other interfaces are supported as well (see documentation)
    with can.Bus(interface='socketcan', channel='vcan0', receive_own_messages=True) as bus:
        # send a message
        message = can.Message(arbitration_id=123, is_extended_id=False,
                              data=[0x11, 0x22, 0x33])
        bus.send(message, timeout=0.2)

        # iterate over received messages
        # for msg in bus:
        #     print(f"{msg.arbitration_id:X}: {msg.data}")

        # or use an asynchronous notifier
        _ = can.Notifier(bus, [can.Logger("recorded.log"), can.Printer()])
        sleep(5)


if __name__ == '__main__':
    # CAN related setup
    # load_layer("can")
    # load_contrib('cansocket')

    conf.contribs['CANSocket'] = {'use-python-can': True}
    load_contrib('cansocket')
    socket = CANSocket(bustype='virtual', channel='vcan0', bitrate=250000)

    # socket = CANSocket(channel='vcan0')

    # multiprocessing setup
    mp.set_start_method('fork')
    processes = [
        GenericECU(socket=socket, vehicle_segment=VehicleSegment.Chassis),
        GenericECU(socket=socket, vehicle_segment=VehicleSegment.PowerTrain),
        GenericECU(socket=socket, vehicle_segment=VehicleSegment.Body),
    ]

    for p in processes:
        p.start()

    for p in processes:
        p.join()
