from can_id_sheet import CANFrame, VehicleSegment, Frequency
from periodic_send_thread import PeriodicSendingTask

from multiprocessing import Process

# from scapy.contrib.cansocket import *
from scapy.contrib.cansocket_python_can import CANSocket
from scapy.layers.can import *
from scapy.sendrecv import *


class GenericECU(Process):
    def __init__(self,
                 socket,  # type: CANSocket
                 vehicle_segment: VehicleSegment
                 ):
        Process.__init__(self)
        self.socket = socket
        self.frames_to_send = [frame for frame in CANFrame if frame.source == vehicle_segment]
        self.name = vehicle_segment.name

        # split frames of this ECU by sending frequency and schedule them on separate cyclic threads
        tasks = []
        ecu_work_description = f'Work description for {self.name} ECU:\n'
        for freq in Frequency:
            frames_for_freq = list(filter(lambda x: x.interval == freq, self.frames_to_send))
            if not frames_for_freq:
                continue
            tasks.append(PeriodicSendingTask(socket=self.socket, frames=frames_for_freq, interval_s=freq.value))
            ecu_work_description += f' - Created cyclic thread which will send {frames_for_freq} every {freq.value}s.\n'
        self.tasks = tasks
        print(ecu_work_description)

    def run(self):
        print(f'Hello from ECU with segment: {self.name} and socket: {self.socket}.')

        for t in self.tasks:
            t.start()

        time.sleep(20)

        for t in self.tasks:
            t.join(timeout=5)
